@@ -11,6 +11,7 @@ __all__ = [
 ]
 
 import builtins
+import collections.abc
 import dataclasses
 import functools
 import io
@@ -40,10 +41,11 @@ logger = logging.getLogger(__name__)
 
 ORIENTATION_ATOL = 1e-5
 
+DType = typing.TypeVar("DType", bound=np.generic)
 T = typing.TypeVar("T")
 
 
-def _all_float_like(seq: typing.Sequence[builtins.float]) -> builtins.bool:
+def _all_float_like(seq: collections.abc.Sequence[builtins.float]) -> builtins.bool:
     return all(isinstance(x, (float, int)) for x in seq)
 
 
@@ -120,9 +122,9 @@ class Cosines:
 
 @dataclasses.dataclass(frozen=True)
 class SortedSlices:
-    slices: typing.Tuple[pydicom.Dataset, ...]
-    indices: typing.Tuple[builtins.int, ...]  # mapping to get original slice order
-    positions: typing.Tuple[builtins.float, ...]
+    slices: builtins.tuple[pydicom.Dataset, ...]
+    indices: builtins.tuple[builtins.int, ...]  # mapping to get original slice order
+    positions: builtins.tuple[builtins.float, ...]
     cosines: Cosines
 
     def __repr__(self) -> builtins.str:
@@ -130,7 +132,7 @@ class SortedSlices:
 
     def __len__(self) -> builtins.int:
         _len = len(self.slices)
-        exn_msg: typing.List[builtins.str] = []
+        exn_msg: builtins.list[builtins.str] = []
         if _len != (ind_len := len(self.indices)):
             exn_msg.append(f"num slices {_len} != num indices {ind_len}")
         if _len != (pos_len := len(self.positions)):
@@ -152,7 +154,7 @@ class SortedSlices:
         imps = (miou.to_f64(sd.ImagePositionPatient) for sd in slice_datasets)
         positions = (np.dot(cs, imp).item() for imp in imps)
         _sorted = typing.cast(
-            typing.Iterable[typing.Tuple[builtins.int, builtins.float]],
+            typing.Iterable[builtins.tuple[builtins.int, builtins.float]],
             sorted(enumerate(positions), key=operator.itemgetter(1)),
         )
         sorted_indices, sorted_positions = miou.unzip(_sorted)
@@ -248,7 +250,7 @@ class SortedSlices:
         values: typing.Sequence[T],
         *,
         atol: builtins.float = ORIENTATION_ATOL,
-    ) -> typing.Tuple[T, ...]:
+    ) -> builtins.tuple[T, ...]:
         # TODO: improve computational efficiency
         # TODO: fix bad init -> bad result
         if not values:
@@ -279,17 +281,17 @@ class SortedSlices:
 
     def _zip(
         self, *args: typing.Iterable[typing.Any]
-    ) -> typing.Iterable[typing.Tuple[typing.Any, ...]]:
+    ) -> typing.Iterable[builtins.tuple[typing.Any, ...]]:
         return zip(self.slices, self.indices, self.positions, *args)
 
 
 @dataclasses.dataclass(frozen=True)
 class DICOMDir:
-    slices: typing.Tuple[pydicom.Dataset, ...]
-    positions: typing.Tuple[builtins.float, ...]
+    slices: builtins.tuple[pydicom.Dataset, ...]
+    positions: builtins.tuple[builtins.float, ...]
     slice_spacing: builtins.float
     affine: npt.NDArray
-    paths: typing.Tuple[miot.PathLike, ...] | None = None
+    paths: builtins.tuple[miot.PathLike, ...] | None = None
 
     def __len__(self) -> builtins.int:
         _len = len(self.slices)
@@ -341,7 +343,7 @@ class DICOMDir:
         defer_size: builtins.str | builtins.int | None = "1 KB",
         extension: builtins.str = ".dcm",
     ) -> DICOMDir:
-        paths: typing.Tuple[miot.PathLike, ...]
+        paths: builtins.tuple[miot.PathLike, ...]
         if (
             isinstance(dicom_path, (builtins.str, pathlib.Path))
             and (_dcm_dir := pathlib.Path(dicom_path)).is_dir()
@@ -357,7 +359,7 @@ class DICOMDir:
             raise ValueError("dicom_dir must be path to a dir. or a list of dcm paths")
         images = tuple(pydicom.dcmread(path, defer_size=defer_size) for path in paths)
         return cls.from_datasets(
-            typing.cast(typing.List[pydicom.Dataset], images),
+            typing.cast(builtins.list[pydicom.Dataset], images),
             paths=paths,
             max_nonuniformity=max_nonuniformity,
             fail_outside_max_nonuniformity=fail_outside_max_nonuniformity,
@@ -367,7 +369,7 @@ class DICOMDir:
     @classmethod
     def from_zipped_stream(
         cls: typing.Type[DICOMDir],
-        data_stream: typing.BinaryIO,
+        data_stream: typing.IO,
         *,
         max_nonuniformity: builtins.float = 5e-4,
         fail_outside_max_nonuniformity: builtins.bool = True,
@@ -396,8 +398,8 @@ class DICOMDir:
     @staticmethod
     def dicom_datasets_from_zip(
         zip_file: zipfile.ZipFile,
-    ) -> typing.List[pydicom.Dataset]:
-        datasets: typing.List[pydicom.Dataset] = []
+    ) -> builtins.list[pydicom.Dataset]:
+        datasets: builtins.list[pydicom.Dataset] = []
         for name in zip_file.namelist():
             if name.endswith("/"):
                 continue  # skip directories
@@ -471,16 +473,16 @@ class DICOMDir:
         return result
 
 
-class DICOMImage(miob.BasicImage):
+class DICOMImage(miob.BasicImage[typing.Any, miot.DType]):  # type: ignore[type-arg]
     @classmethod
     def from_dicomdir(
         cls: typing.Type[DICOMImage],
         dicom_dir: DICOMDir,
         *,
         rescale: builtins.bool | None = None,
-        rescale_dtype: npt.DTypeLike = np.float32,
+        rescale_dtype: typing.Type[miot.DType] | None = None,
         order: typing.Literal["F", "C"] | None = None,
-    ) -> DICOMImage:
+    ) -> DICOMImage[miot.DType]:
         data = cls._merge_slice_pixel_arrays(
             dicom_dir.slices, rescale=rescale, rescale_dtype=rescale_dtype, order=order
         )
@@ -495,10 +497,10 @@ class DICOMImage(miob.BasicImage):
         fail_outside_max_nonuniformity: builtins.bool = True,
         remove_anomalous_images: builtins.bool = True,
         rescale: builtins.bool | None = None,
-        rescale_dtype: npt.DTypeLike = np.float32,
+        rescale_dtype: typing.Type[miot.DType] | None = None,
         order: typing.Literal["F", "C"] | None = None,
         extension: builtins.str = ".dcm",
-    ) -> DICOMImage:
+    ) -> DICOMImage[miot.DType]:
         dicomdir = DICOMDir.from_path(
             dicom_path,
             max_nonuniformity=max_nonuniformity,
@@ -513,17 +515,17 @@ class DICOMImage(miob.BasicImage):
     @classmethod
     def from_zipped_stream(
         cls: typing.Type[DICOMImage],
-        data_stream: typing.BinaryIO,
+        data_stream: typing.IO,
         *,
         max_nonuniformity: builtins.float = 5e-4,
         fail_outside_max_nonuniformity: builtins.bool = True,
         remove_anomalous_images: builtins.bool = True,
         encryption_key: builtins.bytes | builtins.str | None = None,
         rescale: builtins.bool | None = None,
-        rescale_dtype: npt.DTypeLike = np.float32,
+        rescale_dtype: typing.Type[miot.DType] | None = None,
         order: typing.Literal["F", "C"] | None = None,
         **zip_kwargs: typing.Any,
-    ) -> DICOMImage:
+    ) -> DICOMImage[miot.DType]:
         dicomdir = DICOMDir.from_zipped_stream(
             data_stream,
             max_nonuniformity=max_nonuniformity,
@@ -542,11 +544,13 @@ class DICOMImage(miob.BasicImage):
         slices: typing.Sequence[pydicom.Dataset],
         *,
         rescale: builtins.bool | None = None,
-        rescale_dtype: npt.DTypeLike = np.float32,
+        rescale_dtype: typing.Type[miot.DType] | None = None,
         order: typing.Literal["F", "C"] | None = None,
     ) -> npt.NDArray:
         if rescale is None:
             rescale = any(cls._requires_rescaling(d) for d in slices)
+        if rescale and rescale_dtype is None:
+            rescale_dtype = np.float32  # type: ignore[assignment]
 
         first_dataset = slices[0]
         slice_dtype = first_dataset.pixel_array.dtype
